/**
 * Arduino Sketch für SmartHome yourself - Arduino Lehrgang
 * 
 * Tag 18: DIY Sensoren
 * In diesem Sketch verwenden wir Knete für einen Berührungsempfindlichen Taster.
 * Ein GPIO wird so lange eingeschaltet, wie die Knete berührt wird.
 * Ein anderer GPIO wird aktiviert, sobald die gemessene Bodenfeuchtigkeit einen definierten Messwert unterschreitet.
 * 
 * 
 * By Daniel Scheidler - Dezember 2019
 */

const unsigned int RED_LED_PIN = 3;                                // globale Konstante für Pin mit roter LED
const unsigned int GREEN_LED_PIN = 4;                              // globale Konstante für Pin mit grüner LED
const unsigned int TOUCH_BUTTON_PIN = A0;                          // globale Konstante für Analogen Pin mit Knete
const unsigned int MOISTURE_PIN = A3;                              // globale Konstante für Analogen Pin als Feuchtigkeitssensor

const unsigned int LOWER_TOUCH_ANALOG_LIMIT = 1000;                // globale Konstante für Prüfwert auf Berührung.
const unsigned int UPPER_MOISTURE_ANALOG_LIMIT = 235;              // globale Konstante für Prüfwert der Bodenfeuchtigkeit.
                                                                   // Beide Werte sind von vielen Faktoren abhängig. 
                                                                   // Je nach Leitfähigkeit von z.B. Haut, Knete, Erde/Wasser usw.
                                                                   // müssen diese Werte angepasst werden.
                                                            
void setup() {
  Serial.begin(38400);                                             // Serielle Kommunikation starten
  pinMode(RED_LED_PIN, OUTPUT);                                    // PIN von roter LED als OUTPUT festlegen.
  pinMode(GREEN_LED_PIN, OUTPUT);                                  // PIN von grüne LED als OUTPUT festlegen.
  pinMode(TOUCH_BUTTON_PIN, INPUT);                                // PIN von Knete-Taster als INPUT festlegen.
  pinMode(MOISTURE_PIN, INPUT);                                    // PIN von Bodenfeuchtigkeitssensor als INPUT festlegen.
}

void loop() {
  digitalWrite(GREEN_LED_PIN, isButtonPressed() ? HIGH : LOW);     // Wenn Touch-Sensor berührt wird, grüne LED einschalten. Sonst aus.
  digitalWrite(RED_LED_PIN, isMoistureLow() ? HIGH : LOW);         // Wenn Feuchtigkeit unter Grenzwert, rote LED einschalten. Sonst aus. 
}

/**
 * Methode zum überprüfen ob der Touchsensor berührt wird. 
 * 
 * Da ein Pull-Up verwendet wird, liegt der Messwert ohne Berührung bei 1023.
 * Eine Berührung verbindet den TOUCH_BUTTON_PIN mit GND, was einen geringeren Messwert zur Folge hat.
 * Daher wird geprüft, ob der Messwert unter dem definierten Limit liegt. 
 */
boolean isButtonPressed(){
  int touchAnalogPinValue = analogRead(TOUCH_BUTTON_PIN);          // Analogwert des Touch-Sensor-Pins auslesen.
  Serial.println(touchAnalogPinValue);                             // Ausgelesenen Wert auf Seriellem Monitor ausgeben. 

  return touchAnalogPinValue <= LOWER_TOUCH_ANALOG_LIMIT;          // Prüfen ob Analogwert unter dem definierten Limit liegt
                                                                   // und Ergebnis mit true oder false zurückgeben.
}

/**
 * Methode zum überprüfen der Bodenfeuchtigkeit.
 * 
 * Da ein Pull-Up verwendet wird, liegt der Messwert ohne Verbindung 
 * des Kabels am MOISTURE_PIN und dem Kabel an GND bei 1023.
 * Da Wasser Strom leitet, sorgt die Erde je nach Feuchtigkeit 
 * für eine unterschiedlich gute Verbindung der beiden Kabel. 
 * Um so höher die Feuchtigkeit, um so geringer ist also der Messwert. 
 * Daher wird geprüft, ob der Messwert über dem definierten Limit liegt. 
 */
boolean isMoistureLow(){
  int moistureAnalogPinValue = analogRead(MOISTURE_PIN);           // Analogwert des Feuchtigkeits-Sensor-Pins auslesen.
  Serial.println(moistureAnalogPinValue);                          // Ausgelesenen Wert auf Seriellem Monitor ausgeben.

  return moistureAnalogPinValue >= UPPER_MOISTURE_ANALOG_LIMIT;    // Prüfen ob Analogwert über dem definierten Limit liegt
                                                                   // und Ergebnis mit true oder false zurückgeben.
}
